/**
 * Copyright (c) 2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.junit;

import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.ow2.petals.admin.api.artifact.ArtifactState.State;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.ServiceAssembly;
import org.ow2.petals.admin.api.artifact.ServiceUnit;
import org.ow2.petals.admin.api.artifact.SharedLibrary;
import org.ow2.petals.admin.api.artifact.exception.ConfigurationParamComputeException;
import org.ow2.petals.admin.api.artifact.lifecycle.ComponentLifecycle;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.admin.api.exception.ArtifactDeployedException;
import org.ow2.petals.admin.api.exception.ArtifactNotDeployedException;
import org.ow2.petals.admin.api.exception.ArtifactNotFoundException;
import org.ow2.petals.admin.api.exception.ArtifactStartedException;
import org.ow2.petals.admin.api.exception.ArtifactStoppedException;
import org.ow2.petals.admin.api.exception.ArtifactUnsupportedOperationException;
import org.ow2.petals.admin.topology.Container;

/**
 * 
 * @author "Mathieu CARROLLE - EBM WebSourcing"
 */
public class ComponentLifecycleMock extends AbstractArtifactStateLifecycleMock<Component>
        implements ComponentLifecycle {

    public ComponentLifecycleMock(final Component c, final Container container) {
        super(c, container);
    }

    @Override
    public Component getComponent() {
        return this.artifact;
    }

    @Override
    public void deploy(URL artifactUrl, boolean skipInstall)
            throws ArtifactDeployedException, ArtifactAdministrationException {
        super.deploy(artifactUrl);

        // if we skipped the install, then it is in the loaded state and not shutdown
        if (skipInstall) {
            this.artifact.setState(State.LOADED);
        } else {
            setParameters();

            install();
        }
    }

    @Override
    public void updateStateAndParameters() throws ArtifactNotFoundException, ArtifactAdministrationException {
        Component found = findArtefactInContainer();
        if (found != null) {
            this.artifact.getParameters().clear();
            this.artifact.getParameters().putAll(found.getParameters());
        } else {
            throw new ArtifactNotFoundException(artifact.getType(), artifact.getName(), artifact.getVersion());
        }
    }

    @Override
    protected void registerArtefactInContainerForDeploy(Component component) throws ArtifactAdministrationException {
        for (SharedLibrary sl : component.getSharedLibraries()) {
            List<SharedLibrary> sharedLibraries = container.getSharedLibraries();
            if (!sharedLibraries.contains(sl)) {
                throw new ArtifactAdministrationException("Missing SL " + sl + " needed by component");
            }
        }

        this.container.addComponent(artifact);
    }

    @Override
    public void install() throws ArtifactNotFoundException, ArtifactAdministrationException {
        try {
            Component component = findArtefactInContainer();
            if (component == null) {
                throw new ArtifactNotFoundException(artifact.getType(), artifact.getName(), artifact.getVersion());
            }

            setState(State.SHUTDOWN);
        } catch (ArtifactStartedException | ArtifactStoppedException | ArtifactNotDeployedException e) {
            throw new AssertionError("impossible", e);
        }
    }

    @Override
    public void uninstall()
            throws ArtifactNotDeployedException, ArtifactNotFoundException, ArtifactAdministrationException {
        checkUninstall(findArtefactInContainer());

        try {
            setState(State.LOADED);
        } catch (ArtifactStoppedException | ArtifactDeployedException e) {
            throw new AssertionError("impossible", e);
        }
    }

    @Override
    protected Component findArtefactInContainer() {
        for (Component existingComponent : this.container.getComponents()) {
            // the real implementation only use getName() from the current artifact
            if (existingComponent.getName().equals(this.getComponent().getName())) {
                return existingComponent;
            }
        }
        return null;
    }

    @Override
    protected void removeArtefactInContainerForUndeploy(Component artifact) throws ArtifactAdministrationException {
        checkUninstall(artifact);

        this.container.removeComponent(artifact);
    }

    private void checkUninstall(Component artifact) throws ArtifactAdministrationException {
        for (ServiceAssembly sa : this.container.getServiceAssemblies()) {
            for (ServiceUnit su : sa.getServiceUnits()) {
                if (artifact.getName().equals(su.getTargetComponent())) {
                    throw new ArtifactAdministrationException(
                            "SU " + su.getName() + " still deployed on this component");
                }
            }
        }
    }

    @Override
    public void reloadConfiguration()
            throws ArtifactNotFoundException, ArtifactUnsupportedOperationException, ArtifactAdministrationException {
        Component found = findArtefactInContainer();
        if (found == null) {
            throw new ArtifactNotFoundException(this.artifact.getType(), this.artifact.getName(),
                    this.artifact.getVersion());
        }
    }

    @Override
    public void setParameters() throws ArtifactNotFoundException, ArtifactAdministrationException {
        Component found = findArtefactInContainer();
        if (found == null) {
            throw new ArtifactNotFoundException(this.artifact.getType(), this.artifact.getName(),
                    this.artifact.getVersion());
        }

        // let's consider that only the container's component parameters are valid
        Properties installerParameters = found.getParameters();
        if (installerParameters != null && !installerParameters.isEmpty()) {
            Map<String, String> existingConfParams = new HashMap<>();
            for (String property : installerParameters.stringPropertyNames()) {
                existingConfParams.put(property, installerParameters.getProperty(property));
            }

            Properties parameters;
            try {
                parameters = this.artifact.configure(existingConfParams);
            } catch (ConfigurationParamComputeException e) {
                throw new ArtifactAdministrationException(e);
            }

            if (parameters != null) {
                for (String property : parameters.stringPropertyNames()) {
                    // if it's not contained, it means it does not exists!
                    if (!installerParameters.containsKey(property)) {
                        throw new ArtifactAdministrationException(
                                "Unknown or unsettable parameter in configuration MBean: " + property);
                    }
                    found.getParameters().put(property, parameters.getProperty(property));
                }
            }
        } else {
            if (!this.artifact.getParameters().isEmpty()) {
                throw new ArtifactAdministrationException(
                        "Impossible to set configuration parameters for this component (no configuration MBean)");
            }
        }
    }
}