/**
 * Copyright (c) 2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.junit;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.ow2.petals.admin.api.ContainerAdministration;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.Logger;
import org.ow2.petals.admin.api.artifact.ServiceAssembly;
import org.ow2.petals.admin.api.artifact.SharedLibrary;
import org.ow2.petals.admin.api.exception.ContainerAdministrationException;
import org.ow2.petals.admin.api.exception.InvalidLogLevelException;
import org.ow2.petals.admin.api.exception.LoggerNotFoundException;
import org.ow2.petals.admin.junit.exception.NoDomainRegisteredException;
import org.ow2.petals.admin.topology.Container;
import org.ow2.petals.admin.topology.Domain;

/**
 * <p>
 * A mock of the {@link ContainerAdministration} API.
 * </p>
 * <p>
 * This mock provides only one method requiring a connection: {@link #getSystemInfo()}.
 * </p>
 * 
 * @author "Mathieu CARROLLE - EBM WebSourcing"
 * 
 */
public class ContainerAdministrationMock implements ContainerAdministration {

    public static final String INVALID_LOGGER_NAME = "invalid.logger";

    public static final String INVALID_LOG_LEVEL = "INVALID_LEVEL";

    public static final String ROOT_LOGGER_NAME = "";

    public static final String ROOT_LEVEL = "FINEST";

    public static final String LOGGER1_LOGGER_NAME = "logger.1";

    public static final String LOGGER1_LEVEL = "FINEST";

    public static final String LOGGER2_LOGGER_NAME = "logger.2";

    public static final String LOGGER2_LEVEL = "INFO";

    private final Map<String, String> loggers = new HashMap<>();

    private final PetalsAdministrationMock admin;

    public ContainerAdministrationMock(final PetalsAdministrationMock admin) {
        this.admin = admin;
        this.loggers.put(ROOT_LOGGER_NAME, ROOT_LEVEL);
        this.loggers.put(LOGGER1_LOGGER_NAME, LOGGER1_LEVEL);
        this.loggers.put(LOGGER2_LOGGER_NAME, LOGGER2_LEVEL);
    }

    @Override
    public String getSystemInfo() throws ContainerAdministrationException {
        if (!admin.isConnected()) {
            throw new ContainerAdministrationException("No connection establised");
        }
        return admin.getApi().getSystemInfo();
    }

    @Override
    public void stopContainer() throws ContainerAdministrationException {
        // TODO Auto-generated method stub

    }

    @Override
    public void shutdownContainer() throws ContainerAdministrationException {
        // TODO Auto-generated method stub

    }

    @Override
    public Domain getTopology(String securityPassPhrase, boolean retrieveJbiArtifacts)
            throws ContainerAdministrationException {
        return getTopology(null, securityPassPhrase, retrieveJbiArtifacts);
    }

    @Override
    public Domain getTopology(final String containerFilterRegexp, final String securityPassPhrase,
            final boolean retrieveJbiArtifacts) throws ContainerAdministrationException {
        if (!admin.isConnected()) {
            throw new ContainerAdministrationException("No connection establised");
        } else {
            if (admin.getApi().isNullDomain()) {
                return null;
            } else if (admin.getApi().getDomain() == null) {
                throw new ContainerAdministrationException(new NoDomainRegisteredException());
            } else {
                final boolean isOkForSensibleInfo = this.isSecurityTopologyPassphraseOk(securityPassPhrase);
                if (retrieveJbiArtifacts && !isOkForSensibleInfo) {
                    throw new ContainerAdministrationException("Invalid passphrase");
                }
                final Domain domainToReturn = new Domain(admin.getApi().getDomain().getName());
                for (final Container container : admin.getApi().getDomain().getContainers()) {
                    if (containerFilterRegexp == null || container.getContainerName().matches(containerFilterRegexp)) {
                        final Container containerToReturn = new Container(container.getContainerName(),
                                container.getHost(), container.getPorts(),
                                isOkForSensibleInfo ? container.getJmxUsername() : null,
                                isOkForSensibleInfo ? container.getJmxPassword() : null, container.getState());
                        domainToReturn.addContainers(containerToReturn);
                        if (retrieveJbiArtifacts) {
                            for (final SharedLibrary sharedLibrary : container.getSharedLibraries()) {
                                containerToReturn.addSharedLibrary(sharedLibrary.copy());
                            }
                            for (final Component component : container.getComponents()) {
                                containerToReturn.addComponent(component.copy());
                            }
                            for (final ServiceAssembly serviceAssembly : container.getServiceAssemblies()) {
                                containerToReturn.addServiceAssembly(serviceAssembly.copy());
                            }
                        }
                    }
                }
                return domainToReturn;
            }
        }
    }

    private boolean isSecurityTopologyPassphraseOk(final String securityPassPhrase)
            throws ContainerAdministrationException {

        // Check security
        final boolean isSecurityOk;
        if (securityPassPhrase == null || securityPassPhrase.isEmpty()) {
            isSecurityOk = false;
        } else {
            // TODO: The property name of the pass-phrase should be mutualized with the Petals µKernel
            final String containerTopologyPassPhrase = this.getServerProperties()
                    .getProperty("petals.topology.passphrase");
            if (containerTopologyPassPhrase == null || containerTopologyPassPhrase.isEmpty()
                    || !containerTopologyPassPhrase.equals(securityPassPhrase)) {
                isSecurityOk = false;
            } else {
                isSecurityOk = true;
            }
        }
        return isSecurityOk;
    }

    @Override
    public Properties getServerProperties() throws ContainerAdministrationException {
        if (!admin.isConnected()) {
            throw new ContainerAdministrationException("No connection establised");
        } else {
            return admin.getLocalConfiguration();
        }
    }

    @Override
    public void changeLoggerLevel(String loggerName, String loggerLevel)
            throws LoggerNotFoundException, InvalidLogLevelException, ContainerAdministrationException {
        if (INVALID_LOGGER_NAME.equals(loggerName)) {
            throw new LoggerNotFoundException(loggerName);
        }

        if (INVALID_LOG_LEVEL.equals(loggerLevel)) {
            throw new InvalidLogLevelException(loggerLevel);
        }

        this.loggers.put(loggerName, loggerLevel);
    }

    @Override
    public List<Logger> getLoggers() throws ContainerAdministrationException {
        final List<Logger> loggers = new LinkedList<>();
        for (final Map.Entry<String, String> logger : this.loggers.entrySet()) {
            loggers.add(new Logger(logger.getKey(), logger.getValue()));
        }
        return loggers;
    }

    @Override
    public void attachContainer(final String targetDomainName, final String targetContainerHost,
            final int targetContainerPort, final String targetContainerUser, final String targetContainerPwd,
            final String securityPassPhrase) throws ContainerAdministrationException {
        // NOP
    }

    @Override
    public void detachContainer() throws ContainerAdministrationException {
        // NOP
    }

}
